@@ -1,12 +1,14 @@
 """ Representation class for CDB data
 """
-
 import dill
+import json
 import logging
+import aiofiles
 import numpy as np
-from typing import Dict, Set
+from typing import Dict, Set, Optional, List, Union, cast, no_type_check
 from functools import partial
 
+from medcat.utils.hasher import Hasher
 from medcat.utils.matutils import unitvec
 from medcat.utils.ml_utils import get_lr_linking
 from medcat.config import Config, weighted_average, workers
@@ -50,26 +52,26 @@ class CDB(object):
     """
     log = logging.getLogger(__name__)
 
-    def __init__(self, config):
+    def __init__(self, config: Config) -> None:
         self.config = config
-        self.name2cuis = {}
-        self.name2cuis2status = {}
+        self.name2cuis: Dict = {}
+        self.name2cuis2status: Dict = {}
 
-        self.snames = set()
+        self.snames: Set = set()
 
-        self.cui2names = {}
-        self.cui2snames = {}
-        self.cui2context_vectors = {}
-        self.cui2count_train = {}
-        self.cui2info = {}
-        self.cui2tags = {} # Used to add custom tags to CUIs
-        self.cui2type_ids = {}
-        self.cui2preferred_name = {}
-        self.cui2average_confidence = {}
-        self.name2count_train = {}
-        self.name_isupper = {}
+        self.cui2names: Dict = {}
+        self.cui2snames: Dict = {}
+        self.cui2context_vectors: Dict = {}
+        self.cui2count_train: Dict = {}
+        self.cui2info: Dict = {}
+        self.cui2tags: Dict = {} # Used to add custom tags to CUIs
+        self.cui2type_ids: Dict = {}
+        self.cui2preferred_name: Dict = {}
+        self.cui2average_confidence: Dict = {}
+        self.name2count_train: Dict = {}
+        self.name_isupper: Dict = {}
 
-        self.addl_info = {
+        self.addl_info: Dict= {
                 'cui2icd10': {},
                 'cui2opcs4': {},
                 'cui2ontologies': {},
@@ -80,10 +82,10 @@ class CDB(object):
                 'cui2group': {},
                 # Can be extended with whatever is necessary
                 }
-        self.vocab = {} # Vocabulary of all words ever in our cdb
+        self.vocab: Dict = {} # Vocabulary of all words ever in our cdb
         self._optim_params = None
 
-    def get_name(self, cui):
+    def get_name(self, cui: str) -> str:
         r''' Returns preferred name if it exists, otherwise it will return
         the logest name assigend to the concept.
 
@@ -93,18 +95,18 @@ class CDB(object):
 
         name = cui # In case we do not find anything it will just return the CUI
 
-        if cui in self.cui2preferred_name:
+        if cui in self.cui2preferred_name and self.cui2preferred_name[cui]:
             name = self.cui2preferred_name[cui]
-        elif cui in self.cui2names:
+        elif cui in self.cui2names and self.cui2names[cui]:
             name = " ".join(str(max(self.cui2names[cui], key=len)).split(self.config.general.get('separator', '~'))).title()
 
         return name
 
-    def update_cui2average_confidence(self, cui, new_sim):
+    def update_cui2average_confidence(self, cui: str, new_sim: float) -> None:
         self.cui2average_confidence[cui] = (self.cui2average_confidence.get(cui, 0) * self.cui2count_train.get(cui, 0) + new_sim) / \
                                             (self.cui2count_train.get(cui, 0) + 1)
 
-    def remove_names(self, cui: str, names: Dict):
+    def remove_names(self, cui: str, names: Dict) -> None:
         r''' Remove names from an existing concept - efect is this name will never again be used to link to this concept.
         This will only remove the name from the linker (namely name2cuis and name2cuis2status), the name will still be present everywhere else.
         Why? Because it is bothersome to remove it from everywhere, but
@@ -139,7 +141,7 @@ class CDB(object):
                         elif self.name2cuis2status[name][_cui] == 'P':
                             self.name2cuis2status[name][_cui] = 'PD'
 
-    def add_names(self, cui: str, names: Dict, name_status: str='A', full_build: bool=False):
+    def add_names(self, cui: str, names: Dict, name_status: str = 'A', full_build: bool = False) -> None:
         r''' Adds a name to an existing concept.
 
         Args:
@@ -162,7 +164,14 @@ class CDB(object):
 
         self.add_concept(cui=cui, names=names, ontologies=set(), name_status=name_status, type_ids=set(), description='', full_build=full_build)
 
-    def add_concept(self, cui: str, names: Dict, ontologies: set, name_status: str, type_ids: Set[str], description: str, full_build: bool=False):
+    def add_concept(self,
+                    cui: str,
+                    names: Dict,
+                    ontologies: set,
+                    name_status: str,
+                    type_ids: Set[str],
+                    description: str,
+                    full_build: bool = False) -> None:
         r'''
         Add a concept to internal Concept Database (CDB). Depending on what you are providing
         this will add a large number of properties for each concept.
@@ -274,7 +283,7 @@ class CDB(object):
                 else:
                     self.addl_info['type_id2cuis'][type_id] = {cui}
 
-    def add_addl_info(self, name, data, reset_existing=False):
+    def add_addl_info(self, name: str, data: Dict, reset_existing: bool = False) -> None:
         r''' Add data to the addl_info dictionary. This is done in a function to
         not directly access the addl_info dictionary.
 
@@ -291,12 +300,17 @@ class CDB(object):
 
         self.addl_info[name].update(data)
 
-    def update_context_vector(self, cui, vectors, negative=False, lr=None, cui_count=0):
+    def update_context_vector(self,
+                              cui: str,
+                              vectors: Dict[str, np.ndarray],
+                              negative: bool = False,
+                              lr: Optional[float] = None,
+                              cui_count: int = 0) -> None:
         r''' Add the vector representation of a context for this CUI.
 
         cui (`str`):
             The concept in question.
-        vectors (`Dict[str, np.array]`):
+        vectors (`Dict[str, numpy.ndarray]`):
             Vector represenation of the context, must have the format: {'context_type': np.array(<vector>), ...}
             context_type - is usually one of: ['long', 'medium', 'short']
         negative (`bool`, defaults to `False`):
@@ -350,8 +364,8 @@ class CDB(object):
             # Increase counter only for positive examples
             self.cui2count_train[cui] += 1
 
-    def save(self, path):
-        r''' Saves model to file (in fact it saves vairables of this class).
+    def save(self, path: str) -> None:
+        r''' Saves model to file (in fact it saves variables of this class).
 
         Args:
             path (`str`):
@@ -364,8 +378,22 @@ class CDB(object):
             to_save['cdb'] = {k:v for k,v in self.__dict__.items() if k != 'config'}
             dill.dump(to_save, f)
 
+    async def save_async(self, path: str) -> None:
+        r''' Async version of saving model to file (in fact it saves variables of this class).
+
+        Args:
+            path (`str`):
+                Path to a file where the model will be saved
+        '''
+        async with aiofiles.open(path, 'wb') as f:
+            to_save = {
+                'config': self.config.__dict__,
+                'cdb': {k: v for k, v in self.__dict__.items() if k != 'config'}
+            }
+            await f.write(dill.dumps(to_save))
+
     @classmethod
-    def load(cls, path, config=None):
+    def load(cls, path: str, config: Optional[Config] = None) -> "CDB":
         r''' Load and return a CDB. This allows partial loads in probably not the right way at all.
 
         Args:
@@ -376,7 +404,7 @@ class CDB(object):
             # Again no idea
             data = dill.load(f)
             if config is None:
-                config = Config.from_dict(data['config'])
+                config = cast(Config, Config.from_dict(data['config']))
                 cls._ensure_backward_compatibility(config)
 
             # Create an instance of the CDB (empty)
@@ -389,7 +417,8 @@ class CDB(object):
 
         return cdb
 
-    def import_old_cdb_vectors(self, cdb):
+    @no_type_check
+    def import_old_cdb_vectors(self, cdb: "CDB") -> None:
         # Import context vectors
         for cui in self.cui2names: # Loop through all CUIs in the current CDB
             if cui in cdb.cui2context_vec:
@@ -402,7 +431,8 @@ class CDB(object):
 
                 self.cui2count_train[cui] = cdb.cui_count[cui]
 
-    def import_old_cdb(self, cdb, import_vectors=True):
+    @no_type_check
+    def import_old_cdb(self, cdb: "CDB", import_vectors: bool = True) -> None:
         r''' Import all data except for cuis and names from an old CDB.
         '''
 
@@ -437,7 +467,7 @@ class CDB(object):
         # Import cui 2 ontologies
         self.addl_info['cui2ontologies'] = cdb.cui2ontos
 
-    def import_training(self, cdb, overwrite=True):
+    def import_training(self, cdb: "CDB", overwrite: bool = True) -> None:
         r''' This will import vector embeddings from another CDB. No new concepts will be added.
         IMPORTANT it will not import name maps (cui2names, name2cuis or anything else) only vectors.
 
@@ -463,7 +493,7 @@ class CDB(object):
                 # Increase the vector count
                 self.cui2count_train[cui] = self.cui2count_train.get(cui, 0) + cdb.cui2count_train[cui]
 
-    def reset_cui_count(self, n=10):
+    def reset_cui_count(self, n: int = 10) -> None:
         r''' Reset the CUI count for all concepts that received training, used when starting new unsupervised training
         or for suppervised with annealing.
 
@@ -477,7 +507,7 @@ class CDB(object):
         for cui in self.cui2count_train.keys():
             self.cui2count_train[cui] = n
 
-    def reset_training(self):
+    def reset_training(self) -> None:
         r''' Will remove all training efforts - in other words all embeddings that are learnt
         for concepts in the current CDB. Please note that this does not remove synonyms (names) that were
         potentially added during supervised/online learning.
@@ -486,7 +516,7 @@ class CDB(object):
         self.cui2context_vectors = {}
         self.reset_concept_similarity()
 
-    def filter_by_cui(self, cuis_to_keep):
+    def filter_by_cui(self, cuis_to_keep: Union[List[str], Set[str]]) -> None:
         ''' Subset the core CDB fields (dictionaries/maps). Note that this will potenitally keep a bit more CUIs
         then in cuis_to_keep. It will first find all names that link to the cuis_to_keep and then
         find all CUIs that link to those names and keep all of them.
@@ -556,22 +586,34 @@ class CDB(object):
         self.cui2type_ids = new_cui2type_ids
         self.cui2preferred_name = new_cui2preferred_name
 
-    def print_stats(self):
+    def _make_stats(self):
+        stats = {}
+        stats["Number of concepts"] = len(self.cui2names)
+        stats["Number of names"] = len(self.name2cuis)
+        stats["Number of concepts that received training"] = len([cui for cui in self.cui2count_train if self.cui2count_train[cui] > 0])
+        stats["Number of seen training examples in total"] = sum(self.cui2count_train.values())
+        stats["Average training examples per concept"] = np.average(
+                [self.cui2count_train[cui] for cui in self.cui2count_train if self.cui2count_train[cui] > 0])
+
+        return stats
+
+    def print_stats(self) -> None:
         r'''Print basic statistics for the CDB.
         '''
-        self.log.info("Number of concepts: {:,}".format(len(self.cui2names)))
-        self.log.info("Number of names:    {:,}".format(len(self.name2cuis)))
-        self.log.info("Number of concepts that received training: {:,}".format(len([cui for cui in self.cui2count_train if self.cui2count_train[cui] > 0])))
-        self.log.info("Number of seen training examples in total: {:,}".format(sum(self.cui2count_train.values())))
-        self.log.info("Average training examples per concept:     {:.1f}".format(np.average(
-            [self.cui2count_train[cui] for cui in self.cui2count_train if self.cui2count_train[cui] > 0])))
+        self.log.info(json.dumps(self._make_stats(), indent=2))
 
-    def reset_concept_similarity(self):
+    def reset_concept_similarity(self) -> None:
         r''' Reset concept similarity matrix.
         '''
         self.addl_info['similarity'] = {}
 
-    def most_similar(self, cui, context_type, type_id_filter=[], min_cnt=0, topn=50, force_build=False):
+    def most_similar(self,
+                     cui: str,
+                     context_type: str,
+                     type_id_filter: List[str] = [],
+                     min_cnt: int = 0,
+                     topn: int = 50,
+                     force_build: bool = False) -> Dict:
         r''' Given a concept it will calculate what other concepts in this CDB have the most similar
         embedding.
 
@@ -658,7 +700,7 @@ class CDB(object):
         return res
 
     @staticmethod
-    def _ensure_backward_compatibility(config: Config):
+    def _ensure_backward_compatibility(config: Config) -> None:
         # Hacky way of supporting old CDBs
         weighted_average_function = config.linking['weighted_average_function']
         if callable(weighted_average_function) and getattr(weighted_average_function, "__name__", None) == "<lambda>":
@@ -668,3 +710,14 @@ class CDB(object):
         disabled_comps = config.general.get('spacy_disabled_components', [])
         if 'tagger' in disabled_comps and 'lemmatizer' not in disabled_comps:
             config.general['spacy_disabled_components'].append('lemmatizer')
+
+    def get_hash(self):
+        hasher = Hasher()
+
+        for k,v in self.__dict__.items():
+            if k in ['cui2countext_vectors', 'name2cuis']:
+                hasher.update(v, length=False)
+            elif k != 'config':
+                hasher.update(v, length=True)
+
+        return hasher.hexdigest()
