@@ -7,13 +7,16 @@ import datasets
 
 
 _CITATION = """\
-@misc{kraljevic2020multidomain,
-      title={Multi-domain Clinical Natural Language Processing with MedCAT: the Medical Concept Annotation Toolkit}, 
-      author={Zeljko Kraljevic and Thomas Searle and Anthony Shek and Lukasz Roguski and Kawsar Noor and Daniel Bean and Aurelie Mascio and Leilei Zhu and Amos A Folarin and Angus Roberts and Rebecca Bendayan and Mark P Richardson and Robert Stewart and Anoop D Shah and Wai Keong Wong and Zina Ibrahim and James T Teo and Richard JB Dobson},
-      year={2020},
-      eprint={2010.01165},
-      archivePrefix={arXiv},
-      primaryClass={cs.CL}
+@ARTICLE{Kraljevic2021-ln,
+  title="Multi-domain clinical natural language processing with {MedCAT}: The Medical Concept Annotation Toolkit",
+  author="Kraljevic, Zeljko and Searle, Thomas and Shek, Anthony and Roguski, Lukasz and Noor, Kawsar and Bean, Daniel and Mascio, Aurelie and Zhu, Leilei and Folarin, Amos A and Roberts, Angus and Bendayan, Rebecca and Richardson, Mark P and Stewart, Robert and Shah, Anoop D and Wong, Wai Keong and Ibrahim, Zina and Teo, James T and Dobson, Richard J B",
+  journal="Artif. Intell. Med.",
+  volume=117,
+  pages="102083",
+  month=jul,
+  year=2021,
+  issn="0933-3657",
+  doi="10.1016/j.artmed.2021.102083"
 }
 """
 
@@ -23,15 +26,14 @@ Takes as input a pickled dict of annotated documents from MedCAT. The format sho
 Where entities is the output from medcat.get_entities(<...>)['entities']
 """
 
+
 class MedCATAnnotationsConfig(datasets.BuilderConfig):
     """ BuilderConfig for MedCATAnnotations.
 
         Args:
           **kwargs: keyword arguments forwarded to super.
     """
-
-    def __init__(self, **kwargs):
-        super(MedCATAnnotationsConfig, self).__init__(**kwargs)
+    pass
 
 
 class MedCATAnnotations(datasets.GeneratorBasedBuilder):
@@ -45,7 +47,6 @@ class MedCATAnnotations(datasets.GeneratorBasedBuilder):
         ),
     ]
 
-
     def _info(self):
         return datasets.DatasetInfo(
             description=_DESCRIPTION,
@@ -64,7 +65,7 @@ class MedCATAnnotations(datasets.GeneratorBasedBuilder):
             citation=_CITATION,
         )
 
-    def _split_generators(self, dl_manager):
+    def _split_generators(self, dl_manager): # noqa
         """Returns SplitGenerators."""
         return [
             datasets.SplitGenerator(
@@ -82,9 +83,9 @@ class MedCATAnnotations(datasets.GeneratorBasedBuilder):
             docs = pickle.load(f)
             for doc_id in docs:
                 doc = docs[doc_id]
-                for id, entity in doc['entities'].items():
+                for entity_id, entity in doc['entities'].items():
                     yield "{}|{}".format(doc_id, entity['id']), {
-                            'id': int(id),
+                            'id': int(entity_id),
                             'document_id': str(doc_id),
                             'context_left': "".join(entity['context_left']),
                             'context_right': "".join(entity['context_right']),
