@@ -1,6 +1,7 @@
 from spacy.tokens import Span
 import numpy as np
-from medcat.utils.loggers import basic_logger
+# Why is basic_logger not defined in medcat.utils.loggers?
+from medcat.utils.loggers import basic_logger   # type: ignore
 from medcat.utils.matutils import unitvec
 from medcat.utils.ml_utils import load_hf_tokenizer, build_vocab_from_hf
 from spacy.lang.en.stop_words import STOP_WORDS
@@ -25,13 +26,6 @@ class SpacyCat(object):
             the disambiguation using vectors will be performed. While training is True
             it will not be performed
     """
-    # Convert filters tu sets
-    if TUI_FILTER is not None: # noqa
-        TUI_FILTER = set(TUI_FILTER) # noqa
-    if CUI_FILTER is not None: # noqa
-        CUI_FILTER = set(CUI_FILTER) # noqa
-
-
     def __init__(self, cdb, vocab=None, train=False, force_train=False, tokenizer=None):
         self.cdb = cdb
         self.vocab = vocab
@@ -56,7 +50,6 @@ class SpacyCat(object):
         # Weight drops for average
         self.wdrops = [1.0, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2] + [0.1] * 300
 
-
     def _tok_hf(self, token):
         text = token.text
         return self.hf_tokenizer.tokenize(text)
@@ -69,7 +62,6 @@ class SpacyCat(object):
 
         return [text]
 
-
     def _get_doc_words(self, doc, tkns, span, skip_current=False, skip_words=False):
         """ Get words around a certain token
 
@@ -157,7 +149,6 @@ class SpacyCat(object):
 
         return words, weights
 
-
     def _calc_acc(self, cui, doc, tkns, name=None):
         """ Calculate the accuracy for an annotation
 
@@ -223,7 +214,6 @@ class SpacyCat(object):
         else:
             return -1
 
-
     def add_ncntx_vec(self, cui, words):
         cntx_vecs = []
         for word in words:
@@ -236,7 +226,6 @@ class SpacyCat(object):
 
         self.cdb.add_ncontext_vec(cui, cntx)
 
-
     def _add_cntx_vec(self, cui, doc, tkns, negative=False, lr=None, anneal=None):
         """ Add context vectors for this CUI
 
@@ -308,7 +297,6 @@ class SpacyCat(object):
                               unitvec(self.cdb.cui2context_vec[cui])))
                     log.debug(":::::::::::::::::::::::::::::::::::\n")
 
-
     def _add_ann(self, cui, doc, tkns, acc=-1, name=None, is_disamb=False):
         """ Add annotation to a document
 
@@ -366,7 +354,6 @@ class SpacyCat(object):
 
                 self._cuis.add(cui)
 
-
     def _create_main_ann(self, doc, tuis=None):
         """ Only used for testing. Creates annotation in the spacy ents list
         from all the annotations for this document.
@@ -390,7 +377,6 @@ class SpacyCat(object):
 
         doc.ents = list(doc.ents) + main_anns
 
-
     def __call__(self, doc):
         """ Annotate a document
 
@@ -477,7 +463,6 @@ class SpacyCat(object):
 
         return doc
 
-
     def _train_skip(self, name):
         # Down sampling of frequent terms
         cnt_mult = 1
@@ -498,7 +483,6 @@ class SpacyCat(object):
         else:
             return True
 
-
     def disambiguate(self, to_disamb):
         # Do vector disambiguation only if not training
         log.debug("?"*100)
