@@ -1,13 +1,16 @@
 import re
 import os
 import spacy
+from typing import Any, List, Dict, cast, Iterable, Union, Pattern
 from spacy.tokenizer import Tokenizer
 from spacy.language import Language
+from spacy.tokens import Doc
 from tokenizers import ByteLevelBPETokenizer
 from transformers.models.bert.tokenization_bert_fast import BertTokenizerFast
+from medcat.config import Config
 
 
-def spacy_extended(nlp):
+def spacy_extended(nlp: Language) -> Tokenizer:
     infix_re_list = ('\\.\\.+',
     '''(?<=[A-Za-z]{1})[\-_;\,\/~]+(?=[A-Za-z]{1})|(?<=[0-9]{1})[\-_;\,\/]+(?=[A-Za-z]{1})|(?<=[A-Za-z]{1})[\-_;\,\/]+(?=[0-9]{1})|\d{2,4}[\-\s_\*]\d{1,2}[\-\s_\*]\d{1,2}|\d{1,2}:\d{1,2}:\d{1,2}|\d{1,2}:\d{2}'''
      '…',
@@ -17,8 +20,10 @@ def spacy_extended(nlp):
      '(?<=[[[\\p{Lu}&&\\p{Latin}]||[ЁА-Я]||[ӘӨҮҖҢҺ]||[Α-ΩΆΈΊΌΏΉΎ]||[\\p{Ll}&&\\p{Latin}]||[ёа-я]||[әөүҗңһ]||[α-ωάέίόώήύ]||[\\p{L}&&\\p{Bengali}]||[\\p{L}&&\\p{Hebrew}]||[\\p{L}&&\\p{Arabic}]||[\\p{L}&&\\p{Sinhala}]]])[?";:=,.]*(?:-|–|—|--|---|——|~)(?=[[[\\p{Lu}&&\\p{Latin}]||[ЁА-Я]||[ӘӨҮҖҢҺ]||[Α-ΩΆΈΊΌΏΉΎ]||[\\p{Ll}&&\\p{Latin}]||[ёа-я]||[әөүҗңһ]||[α-ωάέίόώήύ]||[\\p{L}&&\\p{Bengali}]||[\\p{L}&&\\p{Hebrew}]||[\\p{L}&&\\p{Arabic}]||[\\p{L}&&\\p{Sinhala}]]])',
      '(?<=[[[\\p{Lu}&&\\p{Latin}]||[ЁА-Я]||[ӘӨҮҖҢҺ]||[Α-ΩΆΈΊΌΏΉΎ]||[\\p{Ll}&&\\p{Latin}]||[ёа-я]||[әөүҗңһ]||[α-ωάέίόώήύ]||[\\p{L}&&\\p{Bengali}]||[\\p{L}&&\\p{Hebrew}]||[\\p{L}&&\\p{Arabic}]||[\\p{L}&&\\p{Sinhala}]]"])[:<>=/](?=[[[\\p{Lu}&&\\p{Latin}]||[ЁА-Я]||[ӘӨҮҖҢҺ]||[Α-ΩΆΈΊΌΏΉΎ]||[\\p{Ll}&&\\p{Latin}]||[ёа-я]||[әөүҗңһ]||[α-ωάέίόώήύ]||[\\p{L}&&\\p{Bengali}]||[\\p{L}&&\\p{Hebrew}]||[\\p{L}&&\\p{Arabic}]||[\\p{L}&&\\p{Sinhala}]]])')
 
-    prefix_re = spacy.util.compile_prefix_regex(Language.Defaults.prefixes)
-    suffix_re = spacy.util.compile_suffix_regex(Language.Defaults.suffixes)
+    prefix_iter = cast(Iterable[Union[str, Pattern[Any]]], Language.Defaults.prefixes)
+    suffix_iter = cast(Iterable[Union[str, Pattern[Any]]], Language.Defaults.suffixes)
+    prefix_re = spacy.util.compile_prefix_regex(prefix_iter)
+    suffix_re = spacy.util.compile_suffix_regex(suffix_iter)
     infix_re = spacy.util.compile_infix_regex(infix_re_list)
 
 
@@ -30,7 +35,7 @@ def spacy_extended(nlp):
             )
 
 
-def spacy_split_all(nlp, config):
+def spacy_split_all(nlp: Language, config: Config) -> Tokenizer:
 
     token_characters = r'[^A-Za-z0-9\@]'
 
@@ -52,12 +57,12 @@ def spacy_split_all(nlp, config):
 class WordpieceTokenizer(object):
     """Runs WordPiece tokenziation."""
 
-    def __init__(self, vocab, unk_token="[UNK]", max_input_chars_per_word=200):
+    def __init__(self, vocab: Any, unk_token: str = "[UNK]", max_input_chars_per_word: int = 200) -> None:
         self.vocab = vocab
         self.unk_token = unk_token
         self.max_input_chars_per_word = max_input_chars_per_word
 
-    def tokenize(self, text):
+    def tokenize(self, text: str) -> List:
         """Tokenizes a piece of text into its word pieces.
         This uses a greedy longest-match-first algorithm to perform tokenization
         using the given vocabulary.
@@ -72,12 +77,12 @@ class WordpieceTokenizer(object):
         """
 
         # Why is convert_to_unicode undefined?
-        text = convert_to_unicode(text) # noqa
+        text = convert_to_unicode(text) # type: ignore # noqa
 
         output_tokens = []
 
         # Why is whitespace_tokenize undefined?
-        for token in whitespace_tokenize(text): # noqa
+        for token in whitespace_tokenize(text): # type: ignore # noqa
             chars = list(token)
             if len(chars) > self.max_input_chars_per_word:
                 output_tokens.append(self.unk_token)
@@ -112,10 +117,10 @@ class WordpieceTokenizer(object):
 
 class SpacyHFTok(object):
 
-    def __init__(self, w2v):
+    def __init__(self, w2v: Any) -> None:
         self.nlp = spacy.load('en_core_sci_md', disable=['ner', 'parser'])
         self.emb_map = {}
-        self.embs = []
+        self.embs: List = []
         for key in w2v.wv.key_to_index.keys():
             self.emb_map[key] = len(self.embs)
             self.embs.append(w2v[key])
@@ -123,16 +128,16 @@ class SpacyHFTok(object):
         # Add pad
         self.embs.append([0.0] * 300)
 
-    def encode(self, text):
+    def encode(self, text: str) -> 'SpacyHFDoc':
         doc = self.nlp(text)
         return SpacyHFDoc(doc)
 
-    def token_to_id(self, tok):
+    def token_to_id(self, tok: Any) -> Any:
         return self.emb_map.get(tok, len(self.emb_map) - 1)
 
 
 class SpacyHFDoc(object):
-    def __init__(self, doc):
+    def __init__(self, doc: Doc) -> None:
         self.doc = doc
         self.tokens = [x.text for x in self.doc]
         self.offsets = [(x.idx, x.idx+len(x.text)) for x in self.doc]
@@ -142,10 +147,10 @@ class TokenizerWrapperBPE(object):
     '''
     '''
 
-    def __init__(self, hf_tokenizers=None):
+    def __init__(self, hf_tokenizers: Any) -> None:
         self.hf_tokenizers = hf_tokenizers
 
-    def __call__(self, text):
+    def __call__(self, text: str) -> Dict:
         res = self.hf_tokenizers.encode(text)
 
         return {'offset_mapping': res.offsets,
@@ -175,7 +180,7 @@ class TokenizerWrapperBERT(object):
     def __init__(self, hf_tokenizers=None):
         self.hf_tokenizers = hf_tokenizers
 
-    def __call__(self, text):
+    def __call__(self, text: str) -> Dict:
         res = self.hf_tokenizers.encode_plus(text,
                 return_offsets_mapping=True, add_special_tokens=False)
 
@@ -184,12 +189,12 @@ class TokenizerWrapperBERT(object):
                 'tokens':  self.hf_tokenizers.convert_ids_to_tokens(res['input_ids']),
                 }
 
-    def save(self, dir_path, name='bert'):
+    def save(self, dir_path: str, name: str='bert') -> None:
         path = os.path.join(dir_path, name)
         self.hf_tokenizers.save_pretrained(path)
 
     @classmethod
-    def load(cls, dir_path, name='bert', **kwargs):
+    def load(cls, dir_path: str, name: str = 'bert', **kwargs) -> Any:
         tokenizer = cls()
         path = os.path.join(dir_path, name)
         tokenizer.hf_tokenizers = BertTokenizerFast.from_pretrained(path, **kwargs)
